#
# wls_install::control
#
# @summary This defined type allows you to control a admin server or a managed server during s Puppet run.
# It allows you to stop, start or restart a managed server. Here is an example on how you could use this:
# 
# ```puppet
# ora_install::control{'startWLSAdminServer12c':
#   domain_name                 => "Wls12c",
#   server_type                 => 'admin',  # admin|managed
#   target                      => 'Server', # Server|Cluster
#   server                      => 'AdminServer',
#   action                      => 'start',
#   weblogic_home_dir           => "/opt/oracle/middleware12c/wlserver",
#   jdk_home_dir                => "/usr/java/jdk1.7.0_45",
#   weblogic_user               => "weblogic",
#   weblogic_password           => "weblogic1",
#   adminserver_address         => 'localhost',
#   adminserver_port            => 7001,
#   nodemanager_port            => 5556,
#   nodemanager_secure_listener => true,
#   os_user                     => "oracle",
#   os_group                    => "dba",
#   download_dir                => "/data/install",
# }
# ```
#
# @param [Stdlib::Absolutepath] middleware_home_dir
#    Middleware home directory.
#
# @param [Stdlib::Absolutepath] weblogic_home_dir
#    Directory to use as WebLogic home.
#
# @param [Stdlib::Absolutepath] jdk_home_dir
#    The homdirectory for the Java evelopment Kit.
#
# @param [Optional[Stdlib::Absolutepath]] wls_domains_dir
#    The root directory where all domain directories reside in.
#
# @param [String[1]] domain_name
#    The domain name.
#
# @param [Enum['admin', 'managed', 'ohs_standalone']] server_type
#    Type of element you want to control:
#    Valid values are:
#    - admin
#    - managed
#    - ohs_standalone
#
# @param [Enum['Server', 'Cluster']] target
#    Type of target to use:
#    Valid values are:
#     - Server
#     - Cluster
#
# @param [String[1]] server
#    Name of the server you want to control.
#
# @param [String[1]] adminserver_address
#    The address of the admin server.
#    May be a DNS name or a physical IP address.
#
# @param [Integer] adminserver_port
#    The IP port the admin server is listening on.
#
# @param [Boolean] nodemanager_secure_listener
#    Use SSL for listening.
#
# @param [Integer] nodemanager_port
#    The IP port the nodemanager is listening on.
#
# @param [Enum['start', 'running', 'stop', 'abort']] action
#    Action to perform.
#    Valid values:
#    - start
#    - running
#    - stop
#    - abort
#
# @param [String[1]] weblogic_user
#    The base WebLogic username to configure.
#
# @param [Easy_type::Password] weblogic_password
#    The password of the base WebLogic user name.
#
# @param [Boolean] jsse_enabled
#    Determine if you want to enable JSSE security.
#
# @param [Boolean] custom_trust
#    Determine if you want to use a custom trust or not.
#
# @param [Optional[String[1]]] trust_keystore_file
#    File specificiation of the trust keystore.
#
# @param [Optional[Easy_type::Password]] trust_keystore_passphrase
#    Passphrase to use for the trust keystore.
#
# @param [String[1]] os_user
#    The os user used to install and run WebLogic.
#
# @param [String[1]] os_group
#    The os group used to install and run WebLogic.
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
#
# See the file "LICENSE" for the full license governing this code.
#
define wls_install::control (
  Stdlib::Absolutepath           $middleware_home_dir         = lookup({name => 'wls_middleware_home_dir'}),
  Stdlib::Absolutepath           $weblogic_home_dir           = lookup({name => 'wls_weblogic_home_dir'}),
  Stdlib::Absolutepath           $jdk_home_dir                = lookup({name => 'wls_jdk_home_dir'}),
  Optional[
    Stdlib::Absolutepath
  ]                              $wls_domains_dir             = lookup({name =>'wls_domains_dir', default_value => undef}),
  String[1]                      $domain_name                 = lookup({name => 'domain_name'}),
  Enum[
    'admin',
    'managed',
    'ohs_standalone'
  ]                              $server_type                 = 'admin',
  Enum[
    'Server',
    'Cluster'
  ]                              $target                      = 'Server',
  String[1]                      $server                      = 'AdminServer',
  String[1]                      $adminserver_address         = lookup({name => 'domain_adminserver_address', default_value => 'localhost'}),
  Integer                        $adminserver_port            = lookup({name => 'domain_adminserver_port', default_value => 7001}),
  Boolean                        $nodemanager_secure_listener = true,
  Integer                        $nodemanager_port            = lookup({name => 'domain_nodemanager_port', default_value => 5556}),
  Enum[
    'start',
    'running',
    'stop',
    'abort'
  ]                              $action                      = 'start',
  String[1]                      $weblogic_user               = lookup({name => 'wls_weblogic_user', default_value => 'weblogic'}),
  Easy_type::Password            $weblogic_password           = lookup({name => 'domain_wls_password'}),
  Boolean                        $jsse_enabled                = lookup({name => 'wls_jsse_enabled', default_value => false}),
  Boolean                        $custom_trust                = lookup({name => 'wls_custom_trust', default_value => false}),
  Optional[String[1]]            $trust_keystore_file         = lookup({name => 'wls_trust_keystore_file', default_value => undef}),
  Optional[Easy_type::Password]  $trust_keystore_passphrase   = lookup({name => 'wls_trust_keystore_passphrase', default_value => undef}),
  String[1]                      $os_user                     = lookup({name => 'wls_os_user'}),
  String[1]                      $os_group                    = lookup({name => 'wls_os_group'}),
  Stdlib::Absolutepath           $download_dir                = lookup({name => 'wls_download_dir'}),
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                              $logoutput                   = lookup({name => 'logoutput', default_value => 'on_failure'}),
)
{
  if $trust_keystore_passphrase {
    $unwrapped_trust_keystore_passphrase = $trust_keystore_passphrase.unwrap
  } else {
    $unwrapped_trust_keystore_passphrase = undef
  }

  if ( $wls_domains_dir ) {
    $domains_dir =  $wls_domains_dir
  } else {
    $domains_dir = "${middleware_home_dir}/user_projects/domains"
  }

  $domain_dir = "${domains_dir}/${domain_name}"

  easy_type::debug_evaluation() # Show local variable on extended debug

  if ( $server_type in ['admin', 'ohs_standalone'] ) {
    $ohs_standalone_server = ( $server_type == 'ohs_standalone' )
    wls_adminserver{ "${title}:AdminServer":
      ensure                      => $action,
      server_name                 => $server,
      domain_name                 => $domain_name,
      domain_path                 => $domain_dir,
      os_user                     => $os_user,
      weblogic_home_dir           => $weblogic_home_dir,
      weblogic_user               => $weblogic_user,
      weblogic_password           => $weblogic_password.unwrap,
      jdk_home_dir                => $jdk_home_dir,
      nodemanager_address         => $adminserver_address,
      nodemanager_port            => $nodemanager_port,
      nodemanager_secure_listener => $nodemanager_secure_listener,
      jsse_enabled                => $jsse_enabled,
      custom_trust                => $custom_trust,
      trust_keystore_file         => $trust_keystore_file,
      trust_keystore_passphrase   => $unwrapped_trust_keystore_passphrase,
      ohs_standalone_server       => $ohs_standalone_server,
    }
  }
  else {
    wls_managedserver{ "${title}:Server":
      ensure              => $action,
      target              => $target,
      server_name         => $server,
      domain_name         => $domain_name,
      os_user             => $os_user,
      weblogic_home_dir   => $weblogic_home_dir,
      weblogic_user       => $weblogic_user,
      weblogic_password   => $weblogic_password.unwrap,
      jdk_home_dir        => $jdk_home_dir,
      adminserver_address => $adminserver_address,
      adminserver_port    => $adminserver_port,
    }
  }
}
