#
# wls_install::copydomain
#
# @summary This defined type allows you copy a domain to an other WebLogic node.
# It is used in conjunction with ora_install::packdomain to build WebLogic clusters. Here is an example on how you could use this:
# 
# ```puppet
# ora_install::copydomain{"Wls12c":
#   version                => 1212,
#   weblogic_home_dir      => "/opt/oracle/middleware12c/wlserver",
#   middleware_home_dir    => "/opt/oracle/middleware12c",
#   jdk_home_dir           => "/usr/java/jdk1.7.0_45",
#   wls_domains_dir        => "/opt/oracle/domains",
#   wls_apps_dir           => "/opt/oracle/applications",
#   domain_name            => "Wls12c",
#   os_user                => "oracle",
#   os_group               => "dba",
#   download_dir           => "/data/install",
#   log_dir                => "/var/log/weblogic",
#   logoutput             => true,
#   use_ssh                => false,
#   domain_pack_dir        => /mnt/fmw_share,
#   adminserver_address    => "10.10.10.10",
#   adminserver_port       => 7001,
#   weblogic_user          => "weblogic",
#   weblogic_password      => "weblogic1",
#   setinternalappdeploymentondemandenable => false,
#   setconfigbackupenabled                 => true,
#   setarchiveconfigurationcount           => 10,
#   setconfigurationaudittype              => 'logaudit',
# }
# ```
#
# @param [Wls_install::Versions] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
# @param [Stdlib::Absolutepath] middleware_home_dir
#    Middleware home directory.
#
# @param [Stdlib::Absolutepath] weblogic_home_dir
#    Directory to use as WebLogic home.
#
# @param [Stdlib::Absolutepath] jdk_home_dir
#    The homdirectory for the Java evelopment Kit.
#
# @param [Optional[Stdlib::Absolutepath]] wls_domains_dir
#    The root directory where all domain directories reside in.
#
# @param [Optional[Stdlib::Absolutepath]] wls_apps_dir
#    The root directory where all aplication directories reside in.
#
# @param [Boolean] use_ssh
#    Boolean value specifying if you want to use ssh to copy the packed file.
#    You can only use this
#    when the ssh keys are already provided.
#
# @param [Optional[Stdlib::Absolutepath]] domain_pack_dir
#    Directory where to put the packed directory.
#
# @param [String[1]] domain_name
#    The domain name.
#
# @param [String[1]] adminserver_address
#    The address of the admin server.
#    May be a DNS name or a physical IP address.
#
# @param [Integer] adminserver_port
#    The IP port the admin server is listening on.
#
# @param [Optional[Integer]] administration_port
#    The common secure administration port for this WebLogic Server domain.
#    (Requires you to enable the administration port.)
#
# @param [Boolean] administration_port_enabled
#    Specifies whether the domain-wide administration port should be enabled for this WebLogic Server domain.
#    Because the administration port uses SSL, enabling the administration port requires that SSL must be configured for all servers in the domain.
#    The domain-wide administration port enables you to start a WebLogic Server instance in STANDBY state. It also allows you to separate administration traffic from application traffic in your domain. Because all servers in the domain must enable or disable the administration port at once, you configure the default administration port settings at the domain level.
#    If you enable the administration port:
#    The administration port accepts only connections that specify administrator credentials.
#    Connections that specify administrator credentials can use only the administration port.
#    The command that starts managed servers must specify a secure protocol and the administration port: -Dweblogic.management.server=https://admin_server:administration_port
#
# @param [Optional[Integer]] adminserver_ssl_port
#    SSL port to use for the Admin server.
#
# @param [Optional[String[1]]] user_config_file
#    This parameter is not used (yet).
#
# @param [Optional[String[1]]] user_key_file
#    This parameter is not used (yet).
#
# @param [String[1]] weblogic_user
#    The base WebLogic username to configure.
#
# @param [Easy_type::Password] weblogic_password
#    The password of the base WebLogic user name.
#
# @param [String[1]] os_user
#    The os user used to install and run WebLogic.
#
# @param [String[1]] os_group
#    The os group used to install and run WebLogic.
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [Optional[Stdlib::Absolutepath]] log_dir
#    Directory to use for logging.
#
# @param [Enum['dev', 'prod']] server_start_mode
#    Startmode to use.
#    This parameter id directly passed to the unpack and start of the domain.
#    Valid values are:
#    - dev
#    - prod
#
# @param [Optional[String[1]]] wls_domains_file
#    file containing the current domains available on the system.
#
# @param [Optional[String[1]]] copy_script
#    The user provided copy script to use for copying the packed domain.
#
# @param [String[1]] puppet_os_user
#    The Puppet OS user to use.
#
# @param [Boolean] jsse_enabled
#    Determine if you want to enable JSSE security.
#
# @param [Boolean] custom_trust
#    Determine if you want to use a custom trust or not.
#
# @param [Optional[String[1]]] trust_keystore_file
#    File specificiation of the trust keystore.
#
# @param [Optional[Easy_type::Password]] trust_keystore_passphrase
#    Passphrase to use for the trust keystore.
#
# @param [Optional[String[1]]] extra_arguments
#    Extra java arguments to use.
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
#
# See the file "LICENSE" for the full license governing this code.
#
define wls_install::copydomain (
  Wls_install::Versions          $version                     = lookup({name => 'wls_version', default_value => 1111}),
  Stdlib::Absolutepath           $middleware_home_dir         = lookup({name => 'wls_middleware_home_dir'}),
  Stdlib::Absolutepath           $weblogic_home_dir           = lookup({name => 'wls_weblogic_home_dir'}),
  Stdlib::Absolutepath           $jdk_home_dir                = lookup({name => 'wls_jdk_home_dir'}),
  Optional[Stdlib::Absolutepath] $wls_domains_dir             = lookup({name => 'wls_domains_dir', default_value => undef}),
  Optional[Stdlib::Absolutepath] $wls_apps_dir                = lookup({name => 'wls_apps_dir', default_value => undef}),
  Boolean                        $use_ssh                     = true,
  Optional[Stdlib::Absolutepath] $domain_pack_dir             = undef,
  String[1]                      $domain_name                 = lookup({name => 'domain_name'}),
  String[1]                      $adminserver_address         = lookup({name => 'domain_adminserver_address'}),
  Integer                        $adminserver_port            = lookup({name => 'domain_adminserver_port', default_value => 7001}),
  Optional[Integer]              $administration_port         = undef,
  Boolean                        $administration_port_enabled = false,
  Optional[Integer]              $adminserver_ssl_port        = undef,
  Optional[String[1]]            $user_config_file            = lookup({name => 'domain_user_config_file', default_value => undef}),
  Optional[String[1]]            $user_key_file               = lookup({name => 'domain_user_key_file', default_value => undef}),
  String[1]                      $weblogic_user               = lookup({name => 'wls_weblogic_user', default_value => 'weblogic'}),
  Easy_type::Password            $weblogic_password           = lookup({name => 'domain_wls_password', default_value => undef}),
  String[1]                      $os_user                     = lookup({name => 'wls_os_user'}),
  String[1]                      $os_group                    = lookup({name => 'wls_os_group'}),
  Stdlib::Absolutepath           $download_dir                = lookup({name => 'wls_download_dir'}),
  Optional[Stdlib::Absolutepath] $log_dir                     = lookup({name => 'wls_log_dir', default_value => undef}),
  Enum['dev','prod']             $server_start_mode           = 'dev',
  Optional[String[1]]            $wls_domains_file            = undef,
  Optional[String[1]]            $copy_script                 = undef,
  String[1]                      $puppet_os_user              = 'root',
  Boolean                        $jsse_enabled                = lookup({name => 'wls_jsse_enabled', default_value => false}),
  Boolean                        $custom_trust                = lookup({name => 'wls_custom_trust', default_value => false}),
  Optional[String[1]]            $trust_keystore_file         = lookup({name => 'wls_trust_keystore_file', default_value => undef}),
  Optional[Easy_type::Password]  $trust_keystore_passphrase   = lookup({name => 'wls_trust_keystore_passphrase', default_value => undef}),
  Optional[String[1]]            $extra_arguments             = lookup({name => 'wls_install::copydomain::extra_arguments', default_value => undef}),
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                              $logoutput                  = lookup({name => 'logoutput', default_value => 'on_failure'}),
)
{
  $root_group = lookup('wls_install::root_group', String[1])
  if ( $wls_domains_file ){
    $wls_domains_file_location = $wls_domains_file
  } else {
    $wls_domains_file_location = '/etc/wls_domains.dat'
  }

  if ( $wls_domains_dir ) {
    $domains_dir =  $wls_domains_dir
  } else {
    $domains_dir = "${middleware_home_dir}/user_projects/domains"
  }

  $domain_dir = "${domains_dir}/${domain_name}"

  if ( $wls_apps_dir ) {
    $apps_dir =  $wls_apps_dir
  } else {
    $apps_dir = "${middleware_home_dir}/user_projects/applications"
  }

  if ( $version in [1036, 1111, 1211] ) {
    $node_mgr_home = "${weblogic_home_dir}/common/nodemanager"
  } elsif ( $version in [1212, 1213] or $version >= 1221) {
    $node_mgr_home = "${domains_dir}/${domain_name}/nodemanager"
  } else {
    $node_mgr_home = "${weblogic_home_dir}/common/nodemanager"
  }

  if ( $domain_pack_dir ) {
    $remote_or_share_domain_domain_dir = $domain_pack_dir
  } else {
    $remote_or_share_domain_domain_dir = $download_dir
  }

  #
  # Depending on the options we specify we need to connect to a different URL for management 
  # of the domain.
  #
  $connect_url = if $administration_port_enabled == true {
    "t3s://${adminserver_address}:${administration_port}"
  } elsif $administration_port_enabled == false and $adminserver_ssl_port != undef {
    "t3s://${adminserver_address}:${adminserver_ssl_port}"
  } else {
    "t3://${adminserver_address}:${adminserver_port}"
  }

  easy_type::debug_evaluation() # Show local variable on extended debug

  unless ( wls_install::domain_exists("${domains_dir}/${domain_name}") ) {

    notify { "wls_install::wlsdomain ${title} ${domains_dir}/${domain_name} ${version} does not exists": }

    $exec_path = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin"

    if ( $log_dir ) {
      # create all log folders
      unless ( defined(Exec["create ${log_dir} directory"]) ) {
        exec { "create ${log_dir} directory":
          command => "mkdir -p ${log_dir}",
          umask   => '0022',
          unless  => "test -d ${log_dir}",
          user    => $os_user,
          path    => $exec_path,
        }
      }
      unless ( defined(File[$log_dir]) ) {
        file { $log_dir:
          ensure  => directory,
          require => Exec["create ${log_dir} directory"],
          mode    => '0775',
          owner   => $os_user,
          group   => $os_group,
        }
      }
    }

    if ( $domains_dir == "${middleware_home_dir}/user_projects/domains" ) {
      unless ( defined(File['weblogic_domain_folder']) ) {
          file { 'weblogic_domain_folder':
            ensure => directory,
            path   => "${middleware_home_dir}/user_projects",
            mode   => '0775',
            owner  => $os_user,
            group  => $os_group,
          }
        File['weblogic_domain_folder'] -> File[$domains_dir]
      }
    }

    unless ( defined(File[$domains_dir]) ) {
      file { $domains_dir:
        ensure => directory,
        mode   => '0775',
        owner  => $os_user,
        group  => $os_group,
      }
    }

    if ( $apps_dir ) {
      unless ( defined(File[$apps_dir]) ) {
        file { $apps_dir:
          ensure => directory,
          mode   => '0775',
          owner  => $os_user,
          group  => $os_group,
        }
      }
    }

    # copy domain from the adminserver to the this node ( with scp and without passwords )
    if ( $copy_script ) {
      exec { "copy with script domain jar ${domain_name}":
        command   => "${copy_script} ${domain_name}.jar ${download_dir}/domain_${domain_name}.jar",
        path      => $exec_path,
        user      => $os_user,
        group     => $os_group,
        logoutput => $logoutput,
        before    =>  Exec["unpack ${domain_name}"],
      }
      $copy_action = "copy with script domain jar ${domain_name}"

    } elsif ( $use_ssh  ) {
      exec { "ssh copy domain jar ${domain_name}":
        command   => "scp -oStrictHostKeyChecking=no -oCheckHostIP=no ${os_user}@${adminserver_address}:${remote_or_share_domain_domain_dir}/domain_${domain_name}.jar ${download_dir}/domain_${domain_name}.jar",
        path      => $exec_path,
        user      => $os_user,
        group     => $os_group,
        logoutput => $logoutput,
        before    =>  Exec["unpack ${domain_name}"]
      }
      $copy_action = "ssh copy domain jar ${domain_name}"

    } else {
      archive { "${download_dir}/domain_${domain_name}.jar":
        ensure  => present,
        cleanup => false,
        extract => false,
        source  => "${remote_or_share_domain_domain_dir}/domain_${domain_name}.jar",
        user    => $os_user,
        group   => $os_group,
        before  =>  Exec["unpack ${domain_name}"]
      }
      #
      # In some cases archive doesn;t set the correct user and group
      #
      ->file { "${download_dir}/domain_${domain_name}.jar":
        owner => $os_user,
        group => $os_group,
      }
      $copy_action = "local copy domain jar ${domain_name}"
    }

    $app_dir_arg = $apps_dir ? {
      undef      => '',
      default    => "-app_dir=${apps_dir}/${domain_name}"
    }

    $unpack_command = "-domain=${domains_dir}/${domain_name} -template=${download_dir}/domain_${domain_name}.jar ${app_dir_arg} -server_start_mode=${server_start_mode} -log=${download_dir}/domain_${domain_name}.log -log_priority=INFO"

    if ( $version >= 1221 ) {
      $bin_dir = "${middleware_home_dir}/oracle_common/common/bin/unpack.sh"
      $wlst_dir = "${middleware_home_dir}/oracle_common/common/bin/wlst.sh"
    } else {
      $bin_dir = "${weblogic_home_dir}/common/bin/unpack.sh"
      $wlst_dir = "${weblogic_home_dir}/common/bin/wlst.sh"
    }

    # On newer WebLogic versions domain creation also create's a template nodemanager.properties
    # because we create the correct nodemanager.properties later in the process by wlsInstall::nodemanager
    # we remove it over here.
    exec { "unpack ${domain_name}":
      command     => "${bin_dir} ${unpack_command} -user_name=${weblogic_user} -password=${weblogic_password.unwrap}",
      environment => "JAVA_HOME=${jdk_home_dir}",
      path        => $exec_path,
      user        => $os_user,
      group       => $os_group,
      logoutput   => $logoutput,
      timeout     => 0,
      require     => File[$domains_dir],
    }

    unless defined(File[$wls_domains_file_location]) {
      file { $wls_domains_file_location:
        ensure => 'present',
        mode   => '0600',
        owner  => 'root',
        group  => $root_group,
      }
    }

    file_line { "domain entry for ${domain_name}":
      path    => $wls_domains_file_location,
      line    => "${domain_name}:${domains_dir}/${domain_name}",
      require => Exec["execwlst ${domain_name} ${title}"],
    }

    # the enroll domain.py used by the wlst
    file { "${download_dir}/enroll_domain_${domain_name}.py":
      ensure    => present,
      content   => epp('wls_install/wlst/enrollDomain.py.epp',{
        'weblogic_user' => $weblogic_user,
        'connect_url'   => $connect_url,
        'domains_dir'   => $domains_dir,
        'domain_name'   => $domain_name,
        'node_mgr_home' => $node_mgr_home,
      }),
      replace   => true,
      mode      => '0775',
      show_diff => false,
      owner     => $os_user,
      group     => $os_group,
      backup    => false,
    }

    if ( $custom_trust ) {
      $config = "-Dweblogic.ssl.JSSEEnabled=${jsse_enabled} -Dweblogic.security.SSL.enableJSSE=${jsse_enabled} -Dweblogic.security.TrustKeyStore=CustomTrust -Dweblogic.security.CustomTrustKeyStoreFileName=${trust_keystore_file} -Dweblogic.security.CustomTrustKeystorePassPhrase=${trust_keystore_passphrase.unwrap} ${extra_arguments}"
    }
    else {
      $config = "-Dweblogic.ssl.JSSEEnabled=${jsse_enabled} -Dweblogic.security.SSL.enableJSSE=${jsse_enabled} ${extra_arguments}"
    }
    debug "Using CONFIG_JVM_ARGS=${config}"

    exec { "execwlst ${domain_name} ${title}":
      command     => "${wlst_dir} ${download_dir}/enroll_domain_${domain_name}.py ${weblogic_password.unwrap}  && /bin/rm -f ${domain_dir}/nodemanager/nodemanager.properties",
      environment => [
        "JAVA_HOME=${jdk_home_dir}",
        "CONFIG_JVM_ARGS=${config}"
      ],
      path        => $exec_path,
      user        => $os_user,
      group       => $os_group,
      logoutput   => $logoutput,
      require     => [
        File["${download_dir}/enroll_domain_${domain_name}.py"],
        Exec["unpack ${domain_name}"],
      ],
    }
    cleanup { "${download_dir}/enroll_domain_${domain_name}.py": }
  }
}
