#
# wls_install::ords
#
# @summary Class for install and config Oracle ORDS software.
#
#
# @param [String[1]] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
# @param [String[1]] source
#    Source for the operation.
#
# @param [String[1]] file
#    The name of the ords install file.
#
# @param [Stdlib::Absolutepath] ords_home
#    The ords home path.
#
# @param [String[1]] db_hostname
#    The hostname of the machine where the database is running.
#
# @param [Integer] db_port
#    The port of the machine where the database is running.
#
# @param [String[1]] db_servicename
#    The name of the service where database is running.
#
# @param [String[1]] db_sid
#    The service id of the machine where the database is running.
#
# @param [String[1]] db_username
#    The name  of the database user.
#
# @param [Boolean] migrate_apex_rest
#    Flag defining if APEX REST api should be migrated.
#
# @param [Boolean] plsql_gateway_add
#    Flag defining if plsql gateway should be added to weblogic.
#
# @param [Boolean] rest_services_apex_add
#    Flag defining if apex rest services should be added to weblogic.
#
# @param [Boolean] rest_services_ords_add
#    Flag defining if ords rest services should be added to weblogic.
#
# @param [String[1]] schema_tablespace_default
#    The default tablespace schema.
#
# @param [String[1]] schema_tablespace_temp
#    The temporary tablespace schema.
#
# @param [Integer] standalone_http_port
#    The standalone http port.
#
# @param [Boolean] standalone_static_images
#    The standalone static images.
#
# @param [String[1]] user_public_password
#    The public password of the user.
#
# @param [String[1]] user_tablespace_default
#    The default tablespace of the user.
#
# @param [String[1]] user_tablespace_temp
#    The temporary tablespace of the user.
#
# @param [String[1]] sys_user
#    The system user.
#
# @param [String[1]] sys_password
#    The system user password.
#
# @param [Variant[Enum['on_failure'], Boolean]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
#
# See the file "LICENSE" for the full license governing this code.
#
class wls_install::ords(
  String[1]                            $version,
  String[1]                            $source,
  String[1]                            $file,
  Stdlib::Absolutepath                 $ords_home,
  String[1]                            $db_hostname,
  Integer                              $db_port,
  String[1]                            $db_servicename,
  String[1]                            $db_sid,
  String[1]                            $db_username,
  Boolean                              $migrate_apex_rest,
  Boolean                              $plsql_gateway_add,
  Boolean                              $rest_services_apex_add,
  Boolean                              $rest_services_ords_add,
  String[1]                            $schema_tablespace_default,
  String[1]                            $schema_tablespace_temp,
  Integer                              $standalone_http_port,
  Boolean                              $standalone_static_images,
  String[1]                            $user_public_password,
  String[1]                            $user_tablespace_default,
  String[1]                            $user_tablespace_temp,
  String[1]                            $sys_user,
  String[1]                            $sys_password,
  Variant[Enum['on_failure'], Boolean] $logoutput       = 'on_failure',    # Output of the execs
){
  easy_type::debug_evaluation() # Show local variable on extended debug

  wls_install::ords::software_version { $version:
    source    => $source,
    file      => $file,
    ords_home => $ords_home,
    logoutput => $logoutput,
  }
  -> wls_install::ords::config { $ords_home:
    db_hostname               => $db_hostname,
    db_port                   => $db_port,
    db_servicename            => $db_servicename,
    db_sid                    => $db_sid,
    db_username               => $db_username,
    migrate_apex_rest         => $migrate_apex_rest,
    plsql_gateway_add         => $plsql_gateway_add,
    rest_services_apex_add    => $rest_services_apex_add,
    rest_services_ords_add    => $rest_services_ords_add,
    schema_tablespace_default => $schema_tablespace_default,
    schema_tablespace_temp    => $schema_tablespace_temp,
    standalone_http_port      => $standalone_http_port,
    standalone_static_images  => $standalone_static_images,
    user_public_password      => $user_public_password,
    user_tablespace_default   => $user_tablespace_default,
    user_tablespace_temp      => $user_tablespace_temp,
    sys_user                  => $sys_user,
    sys_password              => $sys_password,
  }
}
