#
# wls_install::software_version
#
# @summary This defined type allows you install the WebLogic software on your system.
# You can use this to install multiple versions on your system.
# 
# Here is an example on how you could use this:
# 
# ```
# ora_install::software_version{'1221':
#   version              => 1221,                       # 1036|1211|1212|1213|1221
#   filename             => 'fmw_12.2.1.0.0_wls.jar',   # 
#   jdk_home_dir         => '/usr/java/jdk1.8.0_45',
#   oracle_base_home_dir => "/opt/oracle",
#   middleware_home_dir  => "/opt/oracle/middleware12c",
#   weblogic_home_dir    => "/opt/oracle/middleware12c/wlserver",
#   os_user              => "oracle",
#   os_group             => "dba",
#   download_dir         => "/data/install",
#   source               => "/vagrant",                 # puppet:///modules/orawls/ | /mnt |
#   logoutput           => true,
# }
# ```
#
# @param [Wls_install::Versions] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
# @param [String[1]] filename
#    The source file to use.
#
# @param [Stdlib::Absolutepath] oracle_base_home_dir
#    A directory to use as Oracle base directory.
#
# @param [Stdlib::Absolutepath] middleware_home_dir
#    Middleware home directory.
#
# @param [Optional[Stdlib::Absolutepath]] weblogic_home_dir
#    Directory to use as WebLogic home.
#
# @param [Optional[Stdlib::Absolutepath]] wls_domains_dir
#    The root directory where all domain directories reside in.
#
# @param [Optional[Stdlib::Absolutepath]] wls_apps_dir
#    The root directory where all aplication directories reside in.
#
# @param [Boolean] fmw_infra
#    Determine if you want to install Fusion Middleware infra.
#
# @param [Stdlib::Absolutepath] jdk_home_dir
#    The homdirectory for the Java evelopment Kit.
#
# @param [String[1]] os_user
#    The os user used to install and run WebLogic.
#
# @param [String[1]] os_group
#    The os group used to install and run WebLogic.
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [String[1]] source
#    Source for the operation.
#
# @param [Optional[String]] java_parameters
#    The (extra) parameters to pass when calling java.
#
# @param [Stdlib::Absolutepath] temp_directory
#    Directory to use for temporary files.
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
#
# See the file "LICENSE" for the full license governing this code.
#
define wls_install::software_version (
  Wls_install::Versions          $version              = 1111,
  String[1]                      $filename             = undef,
  Stdlib::Absolutepath           $oracle_base_home_dir = undef,
  Stdlib::Absolutepath           $middleware_home_dir  = undef,
  Optional[Stdlib::Absolutepath] $weblogic_home_dir    = undef,
  Optional[Stdlib::Absolutepath] $wls_domains_dir      = undef,
  Optional[Stdlib::Absolutepath] $wls_apps_dir         = undef,
  Boolean                        $fmw_infra            = false,
  Stdlib::Absolutepath           $jdk_home_dir         = undef,
  String[1]                      $os_user              = undef,
  String[1]                      $os_group             = undef,
  Stdlib::Absolutepath           $download_dir         = undef,
  String[1]                      $source               = undef,
  Boolean                        $cleanup              = false,
  Optional[Stdlib::Absolutepath] $oracle_inventory_dir = undef,
  Optional[String]               $java_parameters      = '',
  Stdlib::Absolutepath           $temp_directory       = '/tmp',
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                              $logoutput            = lookup({name => 'logoutput', default_value => 'on_failure'}),
) {
  if ( $wls_domains_dir ) {
    # make sure you don't create the middleware home, else root will be owner
    if ( $wls_domains_dir == "${middleware_home_dir}/user_projects/domains" ) {
        $domains_dir = undef
    } else {
        $domains_dir = $wls_domains_dir
    }
  } else {
    $domains_dir = undef
  }
  if ( $wls_apps_dir ) {
    # make sure you dont create the middleware home, else root will be owner
    if ( $wls_apps_dir == "${middleware_home_dir}/user_projects/applications" ) {
        $apps_dir = undef
    } else {
        $apps_dir = $wls_apps_dir
    }
  } else {
    $apps_dir = undef
  }

  if ( $version in [1036, 1111, 1211] ) {
    $silent_template = 'wls_install/weblogic_silent_install.xml.epp'
    $args= {
      'middleware_home_dir' => $middleware_home_dir,
      'weblogic_home_dir'   => $weblogic_home_dir,
    }
  } else {
    #The oracle home location. This can be an existing Oracle Home or a new Oracle Home
    if ( $fmw_infra ) {
      $install_type = 'Fusion Middleware Infrastructure'
    } else {
      $install_type = 'WebLogic Server'
    }
    if $version >= 1221 {
      $new_version = 1221
    } else {
      $new_version = $version
    }
    $silent_template = "wls_install/weblogic_silent_install_${new_version}.rsp.epp"
    $args= {
      'middleware_home_dir' => $middleware_home_dir,
      'install_type'        => $install_type,
    }
  }

  $exec_path = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:"

  case $::kernel {
    'Linux', 'AIX': {
      $ora_inst_path = '/etc'
      $java_statement = "java ${java_parameters}"
    }
    'SunOS': {
      $ora_inst_path = '/var/opt/oracle'
      $java_statement = "java -d64 ${java_parameters}"
    }
    default: {
      fail("Unrecognized operating system ${::kernel}, please use it on a Linux host")
    }
  }

  if ( $source ) {
    $mount_point = $source
  } else {
    $mount_point = 'puppet:///modules/wls_install/'
  }

  $file_ext = regsubst($filename, '.*(\.jar)$', '\1')

  if ( $file_ext == '.jar' ) {
    $jar_file = true
  } else {
    $jar_file = false
  }

  if ( $jar_file ) {
    $cmd_prefix = "${java_statement} -Xmx1024m -Djava.io.tmpdir=${temp_directory} -jar "
  } else {
    $cmd_prefix = ''
  }

  $weblogic_jar_location = "${download_dir}/${filename}"

  if ( $oracle_inventory_dir ) {
    $ora_inventory = "${oracle_inventory_dir}/oraInventory"
  } else {
    $ora_inventory = "${oracle_base_home_dir}/oraInventory"
  }

  wls_install::utils::orainst { "weblogic orainst ${title}":
    ora_inventory_dir => $ora_inventory,
    os_group          => $os_group,
  }

  easy_type::debug_evaluation() # Show local variable on extended debug

  wls_directory_structure{ "weblogic structure ${title}":
    ensure            => present,
    oracle_base_dir   => $oracle_base_home_dir,
    ora_inventory_dir => $ora_inventory,
    download_dir      => $download_dir,
    wls_domains_dir   => $domains_dir,
    wls_apps_dir      => $apps_dir,
    os_user           => $os_user,
    os_group          => $os_group,
  }

  # if weblogic home dir is specified then check that for creates
  if ( $weblogic_home_dir ) {
    $created_dir = $weblogic_home_dir
  } else {
    $created_dir = $middleware_home_dir
  }

  if $::wls_install_homes.dig('installed_features',$middleware_home_dir) == undef {
    debug "Weblogic home ${created_dir} already exists."

    unless defined(Archive["${download_dir}/${filename}"]) {
      archive { "${download_dir}/${filename}":
        ensure   => present,
        cleanup  => false,
        extract  => false,
        source   => "${mount_point}/${filename}",
        temp_dir => $temp_directory,
        user     => $os_user,
        group    => $os_group,
        before   => Exec["install weblogic ${title}"],
        require  => Wls_directory_structure["weblogic structure ${title}"],
      }
      ~> exec { 'Set jar ownership':
        command     => "/usr/bin/chown ${os_user}:${os_group} ${download_dir}/${filename}",
        refreshonly => true
      }
    }

    # de xml used by the wls installer
    file { "${download_dir}/weblogic_silent_install_${title}.xml":
      ensure    => present,
      content   => epp($silent_template,$args),
      replace   => true,
      mode      => '0775',
      owner     => $os_user,
      group     => $os_group,
      backup    => false,
      show_diff => false,
      require   => Wls_directory_structure["weblogic structure ${title}"],
    }

    if ( $version in [1212, 1213] or $version >= 1221) {

      $command = "-silent -responseFile ${download_dir}/weblogic_silent_install_${title}.xml "

      exec { "install weblogic ${title}":
        command     => "${cmd_prefix}${weblogic_jar_location} ${command} -invPtrLoc ${ora_inst_path}/oraInst.loc -ignoreSysPrereqs",
        environment => [
          'JAVA_VENDOR=Sun',
          "JAVA_HOME=${jdk_home_dir}"
        ],
        timeout     => 0,
        creates     => $created_dir,
        path        => $exec_path,
        user        => $os_user,
        group       => $os_group,
        logoutput   => $logoutput,
        require     => [
          Wls_directory_structure["weblogic structure ${title}"],
          Wls_install::Utils::Orainst["weblogic orainst ${title}"],
          File["${download_dir}/weblogic_silent_install_${title}.xml"],
        ],
      }
      # OPatch native lib fix for 64 solaris
      case $::kernel {
        'SunOS': {
          exec { "add -d64 oraparam.ini oracle_common ${title}":
            command   => "sed -e's/JRE_MEMORY_OPTIONS=/JRE_MEMORY_OPTIONS=\"-d64\"/g' ${middleware_home_dir}/oui/oraparam.ini > ${temp_directory}/wls.tmp && mv ${temp_directory}/wls.tmp ${middleware_home_dir}/oui/oraparam.ini",
            unless    => "grep 'JRE_MEMORY_OPTIONS=\"-d64\"' ${middleware_home_dir}/oui/oraparam.ini",
            require   => Exec["install weblogic ${title}"],
            path      => $exec_path,
            user      => $os_user,
            group     => $os_group,
            logoutput => $logoutput,
          }
        }
        default: {} # Do nothin if not on Solaris
      }

    } else {
      exec {"install weblogic ${title}":
        command     => "${cmd_prefix}${weblogic_jar_location} -Djava.io.tmpdir=${temp_directory} -Duser.country=US -Duser.language=en -mode=silent -log=${temp_directory}/wls_${title}.out -log_priority=info -silent_xml=${download_dir}/weblogic_silent_install_${title}.xml",
        environment => [
          'JAVA_VENDOR=Sun',
          "JAVA_HOME=${jdk_home_dir}"
        ],
        creates     => $created_dir,
        timeout     => 0,
        path        => $exec_path,
        user        => $os_user,
        group       => $os_group,
        logoutput   => $logoutput,
        require     => [
          Wls_directory_structure["weblogic structure ${title}"],
          File["${download_dir}/weblogic_silent_install_${title}.xml"],
        ],
      }
    }
    cleanup { ["${download_dir}/${filename}","${download_dir}/weblogic_silent_install_${title}.xml"]:}
  }
}
