#
# wls_install::support::nodemanagerautostart
#
# @summary 
# This defined type allows you to add the nodemanager startup scripts to the system. Here is an example on how you could use this:```puppetora_install::support::nodemanagerautostart{"autostart ${wlsDomainName}":  version      => 1212,  wl_home      => '/opt/oracle/middleware12c/wlserver',  user         => 'oracle',  domain       => 'Wls1212',  domain_path  => '/opt/oracle/middleware12c/user_projects/domains/Wls1212'```
#
# @param [Wls_install::Versions] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
# @param [Stdlib::Absolutepath] wl_home
#    WebLogic home directory.
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
# @param [String[1]] domain
#    Domain name.
#
# @param [String[1]] domain_path
#    Path where the domain inforrmation is stored.
#
# @param [Stdlib::Absolutepath] log_dir
#    Directory to use for logging.
#
# @param [Boolean] jsse_enabled
#    Determine if you want to enable JSSE security.
#
# @param [Boolean] custom_trust
#    Determine if you want to use a custom trust or not.
#
# @param [Optional[String[1]]] trust_keystore_file
#    File specificiation of the trust keystore.
#
# @param [Optional[Easy_type::Password]] trust_keystore_passphrase
#    Passphrase to use for the trust keystore.
#
# @param [Optional[Stdlib::Absolutepath]] systemd_script_location
#    Location to put the systemd script.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define wls_install::support::nodemanagerautostart(
  Wls_install::Versions          $version                   = 1111,
  Stdlib::Absolutepath           $wl_home                   = undef,
  String[1]                      $user                      = 'oracle',
  String[1]                      $domain                    = undef,
  String[1]                      $domain_path               = undef,
  Stdlib::Absolutepath           $log_dir                   = undef,
  Boolean                        $jsse_enabled              = false,
  Boolean                        $custom_trust              = false,
  Optional[String[1]]            $trust_keystore_file       = undef,
  Optional[Easy_type::Password]  $trust_keystore_passphrase = undef,
  Optional[Stdlib::Absolutepath] $systemd_script_location   = undef,
){
  case $version {
    1036, 1111, 1211: {
      $node_mgr_path = "${wl_home}/common/nodemanager"
      $node_mgr_bin_path = "${wl_home}/server/bin"
      $script_name = "nodemanager_${version}"

      if ( $log_dir ) {
        $node_mgr_lck_file = "${log_dir}/nodemanager.log.lck"
      } else {
        $node_mgr_lck_file = "${node_mgr_path}/nodemanager.log.lck"
      }
    }
    1212, 1213, 1221, 12211, 12212, 12213, 12214, 14110 : {
      $node_mgr_path = "${domain_path}/nodemanager"
      $node_mgr_bin_path = "${domain_path}/bin"
      $script_name = "nodemanager_${domain}"

      if ( $log_dir ) {
        $node_mgr_lck_file = "${log_dir}/nodemanager_${domain}.log.lck"
      } else {
        $node_mgr_lck_file = "${node_mgr_path}/nodemanager_${domain}.log.lck"
      }
    }
    default: {
      $node_mgr_path = "${wl_home}/common/nodemanager"
      $node_mgr_bin_path = "${wl_home}/server/bin"
      $script_name = 'nodemanager'

      if ( $log_dir ) {
        $node_mgr_lck_file = "${log_dir}/nodemanager.log.lck"
      } else {
        $node_mgr_lck_file = "${node_mgr_path}/nodemanager.log.lck"
      }
    }
  }

  if ( $custom_trust ) {
    $trust_env = "-Dweblogic.security.TrustKeyStore=CustomTrust -Dweblogic.security.CustomTrustKeyStoreFileName=${trust_keystore_file}"
  } else {
    $trust_env = ''
  }

  if ( $::operatingsystem in ['RedHat', 'CentOS', 'OracleLinux', 'AlmaLinux', 'Rocky'] and $::operatingsystemmajrelease == '7' ) {
    if ( $systemd_script_location ) {
      $location = "${systemd_script_location}/${script_name}"
    } else {
      $location = "/home/${user}/${script_name}"
    }
  } else {
    $location = "/etc/init.d/${script_name}"
  }

  easy_type::debug_evaluation() # Show local variable on extended debug

  file { $location :
    ensure  => present,
    mode    => '0755',
    content => regsubst(epp('wls_install/nodemanager.epp',{
      'version'           => $version,
      'domain'            => $domain,
      'domain_path'       => $domain_path,
      'osfamily'          => $::osfamily,
      'node_mgr_bin_path' => $node_mgr_bin_path,
      'user'              => $user,
      'jsse_enabled'      => $jsse_enabled,
      'trust_env'         => $trust_env,
      'node_mgr_lck_file' => $node_mgr_lck_file}), '\r\n', "\n", 'EMG'),
  }

  $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin'

  case $::operatingsystem {
    'AIX': {
      fail 'autostart not (yet) supported on AIX systems.'
    }
    'RedHat', 'CentOS', 'OracleLinux', 'AlmaLinux', 'Rocky': {
      if ( $::operatingsystemmajrelease == '7') {
        file { "/lib/systemd/system/${script_name}.service" :
          ensure  => present,
          mode    => '0644',
          content => regsubst(epp('wls_install/systemd.epp',{
            'script_name' => $script_name,
            'location'    => $location
          }), '\r\n', "\n", 'EMG'),
          require => File[$location],
        }
        exec { "systemctl daemon-reload ${title}":
          command     => 'systemctl daemon-reload',
          subscribe   => File["/lib/systemd/system/${script_name}.service"],
          refreshonly => true,
          user        => 'root',
          path        => $exec_path,
          logoutput   => true,
        }
        exec { "systemctl enable ${title}":
          command   => "systemctl enable ${script_name}.service",
          require   => [
            Exec["systemctl daemon-reload ${title}"],
            File["/lib/systemd/system/${script_name}.service"]
          ],
          user      => 'root',
          unless    => "systemctl list-units --type service --all | /bin/grep '${script_name}.service'",
          path      => $exec_path,
          logoutput => true,
        }
      }
      else {
        exec { "chkconfig ${script_name}":
          command   => "chkconfig --add ${script_name}",
          require   => File[$location],
          user      => 'root',
          unless    => "chkconfig | /bin/grep '${script_name}'",
          path      => $exec_path,
          logoutput => true,
        }
      }
    }
    'SLES':{
        exec { "chkconfig ${script_name}":
          command   => "chkconfig --add ${script_name}",
          require   => File[$location],
          user      => 'root',
          unless    => "chkconfig | /bin/grep '${script_name}'",
          path      => $exec_path,
          logoutput => true,
        }
    }
    'Ubuntu', 'Debian':{
      exec { "update-rc.d ${script_name}":
        command   => "update-rc.d ${script_name} defaults",
        require   => File[$location],
        user      => 'root',
        unless    => "ls /etc/rc3.d/*${script_name} | /bin/grep '${script_name}'",
        path      => $exec_path,
        logoutput => true,
      }
    }
    default: {
      fail('Unrecognized operating system')
    }
  }
}
