#
# wls_install::utils::forms11gpatch
#
# @summary Type installing forms11gpatch.
# Here is an example on how to use it:
# 
# ``` puppet
# }
# ```
#
# @param [Wls_install::Versions] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
# @param [Stdlib::Absolutepath] weblogic_home_dir
#    Directory to use as WebLogic home.
#
# @param [Stdlib::Absolutepath] middleware_home_dir
#    Middleware home directory.
#
# @param [Stdlib::Absolutepath] oracle_base_home_dir
#    A directory to use as Oracle base directory.
#
# @param [Stdlib::Absolutepath] oracle_home_dir
#    A directory to be used as Oracle home directory for this software.
#
# @param [Stdlib::Absolutepath] jdk_home_dir
#    The homdirectory for the Java evelopment Kit.
#
# @param [String[1]] fmw_file1
#    First file of FMW distribution.
#
# @param [String[1]] os_user
#    The os user used to install and run WebLogic.
#
# @param [String[1]] os_group
#    The os group used to install and run WebLogic.
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [String[1]] source
#    Source for the operation.
#
# @param [Stdlib::Absolutepath] temp_directory
#    Directory to use for temporary files.
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
#
# See the file "LICENSE" for the full license governing this code.
#
define wls_install::utils::forms11gpatch(
  Wls_install::Versions $version              = lookup({name => 'wls_version', default_value => 1111}),
  Stdlib::Absolutepath  $weblogic_home_dir    = lookup({name => 'wls_weblogic_home_dir'}),
  Stdlib::Absolutepath  $middleware_home_dir  = lookup({name => 'wls_middleware_home_dir'}),
  Stdlib::Absolutepath  $oracle_base_home_dir = lookup({name => 'wls_oracle_base_home_dir'}),
  Stdlib::Absolutepath  $oracle_home_dir      = undef,
  Stdlib::Absolutepath  $jdk_home_dir         = lookup({name => 'wls_jdk_home_dir'}),
  String[1]             $fmw_file1            = undef,
  String[1]             $os_user              = lookup({name => 'wls_os_user'}),
  String[1]             $os_group             = lookup({name => 'wls_os_group'}),
  Stdlib::Absolutepath  $download_dir         = lookup({name => 'wls_download_dir'}),
  String[1]             $source               = lookup({name => 'wls_source', default_value => undef}),
  Stdlib::Absolutepath  $temp_directory       = lookup({name => 'wls_temp_dir', default_value => '/tmp'}),
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                     $logoutput            = lookup({name => 'logoutput', default_value => 'on_failure'}),
)
{
  $fmw_product = 'forms_patch'

  $exec_path = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:"
  $ora_inventory = "${oracle_base_home_dir}/oraInventory"

  case $::kernel {
    'Linux': {
      $ora_inst_path = '/etc'
      case $::architecture {
        'i386': {
          $install_dir = 'linux'
        }
        default: {
          $install_dir = 'linux64'
        }
      }
    }
    'SunOS': {
      $ora_inst_path = '/var/opt/oracle'
      case $::architecture {
        'i86pc': {
          $install_dir = 'intelsolaris'
        }
        default: {
          $install_dir = 'solaris'
        }
      }
    }
    default: {
      fail("Unrecognized operating system ${::kernel}, please use it on a Linux host")
    }

  }

  $fmw_silent_response_file = 'wls_install/fmw_silent_forms_patch.rsp.epp'
  if ( $oracle_home_dir ) {
    $oracle_home = $oracle_home_dir
  } else {
    $oracle_home = "${middleware_home_dir}/Oracle_FRM1"
  }

  $create_file_1 = "${download_dir}/${fmw_product}/Disk1"

  if ( $source ) {
    $mount_point = $source
  } else {
    $mount_point = 'puppet:///modules/wls_install/'
  }

  easy_type::debug_evaluation() # Show local variable on extended debug

  file { "${download_dir}/${title}_silent_${fmw_product}.rsp":
    ensure  => present,
    content => epp($fmw_silent_response_file,{
      'oracle_home'         => $oracle_home,
      'middleware_home_dir' => $middleware_home_dir,
    }),
    mode    => '0775',
    owner   => $os_user,
    group   => $os_group,
    backup  => false,
  }

  file { "${download_dir}/${fmw_product}":
    ensure => directory,
    mode   => '0775',
    owner  => $os_user,
    group  => $os_group,
    backup => false,
    before => Archive["${download_dir}/${fmw_file1}"],
  }

  archive { "${download_dir}/${fmw_file1}":
    ensure       => present,
    cleanup      => true,
    creates      => $create_file_1,
    group        => $os_group,
    extract      => true,
    extract_path => "${download_dir}/${fmw_product}",
    require      => File["${download_dir}/${fmw_product}"],
    source       => "${mount_point}/${fmw_file1}",
    temp_dir     => $temp_directory,
    user         => $os_user,
  }

  $command = "-silent -response ${download_dir}/${title}_silent_${fmw_product}.rsp -waitforcompletion"

  exec { "install ${fmw_product} ${title}":
    command     => "/bin/sh -c 'unset DISPLAY;${download_dir}/${fmw_product}/Disk1/install/${install_dir}/runInstaller ${command} -invPtrLoc ${ora_inst_path}/oraInst.loc -ignoreSysPrereqs -jreLoc ${jdk_home_dir} -Djava.io.tmpdir=${temp_directory}'",
    environment => "TEMP=${temp_directory}",
    timeout     => 0,
    path        => $exec_path,
    user        => $os_user,
    group       => $os_group,
    logoutput   => $logoutput,
  }
}
